/**
***     Copyright (C) 2008 - 2010 Sophia Cradle Incorporated.
***     All rights reserved.
***
***     This source code was written by HORIGUCHI Junshi.
***
***     Project         : SophiaFramework
***     Codename        : Graviton
***     Platform        : x86, ARM
***     OS              : BREW 2.0, BREW 2.1, BREW 3.1, BREW 4.0
***     Compiler        : Visual C++ 6.0, Visual C++ .NET 2003, Visual C++ 2005,
***                       ADS 1.2, RVCTB 1.2, RVCTB 3.0,
***                       GNUARM 4.1.1, YAGARTO 4.4.2
***     Package         : Graphical User Interface Package
***     File            : SFYSingleEditWidget.h.hpp
**/

#ifndef __SOPHIAFRAMEWORK_SFYSINGLEEDITWIDGET_HHPP
#define __SOPHIAFRAMEWORK_SFYSINGLEEDITWIDGET_HHPP

#include <SFYResponder/SFYEnvironment.h.hpp>
#include <SFYResponder/SFYWidget/SFYSingleEditWidget.f.hpp>
#include <SFYResponder/SFYWidget/SFYWidget.h.hpp>
#include <SFYResponder/SFYWidget/SFYSingleTextWidget.h.hpp>
#include <SFXGeneral/SFXBaseEditor.h.hpp>
#include <SFXGeneral/SFXEditProperty.h.hpp>

class SFYSingleEditWidget : public SFYWidget {
    SFMSEALRESPONDER(SFYSingleEditWidget)
    SFMRESPONDERINSTANTIATETWO(SFYSingleEditWidget, SFYWidget, SFYResponder)
    public:
        enum CodeEnum {
            CODE_TYPE                           = four_char_code('+', 's', 'e', 'd')
        };
        SFMTYPEDEFTYPE(CodeEnum)
        enum DefaultEnum {
            DEFAULT_WAIT                        = SFYSingleTextWidget::DEFAULT_WAIT,
            DEFAULT_SCROLL                      = SFYSingleTextWidget::DEFAULT_SCROLL,
            DEFAULT_STEP                        = SFYSingleTextWidget::DEFAULT_STEP
        };
        SFMTYPEDEFTYPE(DefaultEnum)
        enum HorizontalEnum {
            HORIZONTAL_LEFT                     = SFYSingleTextWidget::HORIZONTAL_LEFT,
            HORIZONTAL_CENTER                   = SFYSingleTextWidget::HORIZONTAL_CENTER,
            HORIZONTAL_RIGHT                    = SFYSingleTextWidget::HORIZONTAL_RIGHT,
            DEFAULT_HORIZONTAL                  = SFYSingleTextWidget::DEFAULT_HORIZONTAL
        };
        SFMTYPEDEFTYPE(HorizontalEnum)
        enum VerticalEnum {
            VERTICAL_TOP                        = SFYSingleTextWidget::VERTICAL_TOP,
            VERTICAL_MIDDLE                     = SFYSingleTextWidget::VERTICAL_MIDDLE,
            VERTICAL_BOTTOM                     = SFYSingleTextWidget::VERTICAL_BOTTOM,
            DEFAULT_VERTICAL                    = SFYSingleTextWidget::DEFAULT_VERTICAL
        };
        SFMTYPEDEFTYPE(VerticalEnum)
        enum DirectionEnum {
            DIRECTION_LEFT                      = SFYSingleTextWidget::DIRECTION_LEFT,
            DIRECTION_RIGHT                     = SFYSingleTextWidget::DIRECTION_RIGHT,
            DEFAULT_DIRECTION                   = SFYSingleTextWidget::DEFAULT_DIRECTION
        };
        SFMTYPEDEFTYPE(DirectionEnum)

    private:
                SFYSingleTextWidgetSmp          _widget;
                SFXBaseEditorPtr                _editor;
                SFXEditProperty                 _property;
                SFXWideString                   _text;
                UInt16                          _maximum;

    public:
        static  SFYSingleEditWidgetSmp          NewInstance                     (SFCErrorPtr exception = null);
                SFCError                        SetText                         (SFXPathConstRef path, UInt16 id);
                SFCError                        SetText                         (SFXWideStringConstRef param);
                SFXWideStringConstRef           GetText                         (Void) const;
                SFXSizeConstRef                 GetTextSize                     (Void) const;
                Void                            SetTextColor                    (SFXRGBColorConstRef param);
                SFXRGBColorConstRef             GetTextColor                    (Void) const;
                Void                            SetFont                         (AEEFont param);
                AEEFont                         GetFont                         (Void) const;
                SFXSizeConstRef                 GetFontSize                     (Void) const;
                Void                            SetHorizontalAlign              (HorizontalEnum param);
                HorizontalEnum                  GetHorizontalAlign              (Void) const;
                Void                            SetVerticalAlign                (VerticalEnum param);
                VerticalEnum                    GetVerticalAlign                (Void) const;
                SFCError                        SetMaximumLength                (UInt16 param);
                UInt16                          GetMaximumLength                (Void) const;
                Void                            SetInputMode                    (AEETextInputMode param);
                AEETextInputMode                GetInputMode                    (Void) const;
                SFCError                        SetPasswordMode                 (Bool param);
                Bool                            GetPasswordMode                 (Void) const;
                Void                            SetWaitInterval                 (UInt32 param);
                UInt32                          GetWaitInterval                 (Void) const;
                Void                            SetScrollInterval               (UInt32 param);
                UInt32                          GetScrollInterval               (Void) const;
                Void                            SetScrollDirection              (DirectionEnum param);
                DirectionEnum                   GetScrollDirection              (Void) const;
                Void                            SetScrollStep                   (SInt16 param);
                SInt16                          GetScrollStep                   (Void) const;
                Void                            StartScroll                     (Void);
                Void                            StopScroll                      (Void);
                SFCError                        Edit                            (Void);
    protected:
        explicit                                SFYSingleEditWidget             (Void) static_throws;
        virtual                                 ~SFYSingleEditWidget            (Void);
                SFYSingleTextWidgetSmpConstRef  GetWidget                       (Void) const;
        virtual Void                            HandleBoundRequest              (SFXRectanglePtr rectangle) const;
        virtual Void                            HandleBoundOptimize             (SFXRectanglePtr rectangle) const;
        virtual Void                            HandleBoundReal                 (Void);
        virtual Void                            HandleBoundVirtual              (Void);
    private:
                XANDLER_DECLARE_VOIDSTYLE(OnWidgetStyle)
                XALLBACK_DECLARE_SFXEDITOR(OnEditor)
                XALLBACK_DECLARE_SFYDISTRIBUTER(OnDistribute)
                SFCError                        SetTextMaximumLength            (SFXWideStringConstRef text, UInt16 maximum);
                SFCError                        SetWidgetText                   (SFXWideStringConstRef text, Bool password);
                Void                            Relocate                        (Void);
//////////////////////////////////////// Will be deprecated in version 6.0 //////////
    public:
                Void                            SetSFBTextCtlClassID            (AEECLSID param);
                AEECLSID                        GetSFBTextCtlClassID            (Void) const;
                Void                            SetSFBMenuCtlClassID            (AEECLSID param);
                AEECLSID                        GetSFBMenuCtlClassID            (Void) const;
/////////////////////////////////////////////////////////////////////////////////////
};

#include <SFYResponder/SFYWidget/SFYSingleEditWidget.i.hpp>

#endif // __SOPHIAFRAMEWORK_SFYSINGLEEDITWIDGET_HHPP //
