/**
***     Copyright (C) 2007 - 2010 Sophia Cradle Incorporated.
***     All rights reserved.
***
***     This source code was written by HORIGUCHI Junshi.
***
***     Project         : SophiaFramework
***     Codename        : Graviton
***     Platform        : x86, ARM
***     OS              : BREW 2.0, BREW 2.1, BREW 3.1, BREW 4.0
***     Compiler        : Visual C++ 6.0, Visual C++ .NET 2003, Visual C++ 2005,
***                       ADS 1.2, RVCTB 1.2, RVCTB 3.0,
***                       GNUARM 4.1.1, YAGARTO 4.4.2
***     Package         : Graphical User Interface Package
***     File            : SFYSingleTextWidget.h.hpp
**/

#ifndef __SOPHIAFRAMEWORK_SFYSINGLETEXTWIDGET_HHPP
#define __SOPHIAFRAMEWORK_SFYSINGLETEXTWIDGET_HHPP

#include <SFYResponder/SFYEnvironment.h.hpp>
#include <SFYResponder/SFYWidget/SFYSingleTextWidget.f.hpp>
#include <SFYResponder/SFYWidget/SFYWidget.h.hpp>
#include <SFXString/SFXWideString.h.hpp>
#include <SFXStorage/SFXFile/SFXPath.f.hpp>
#include <SFXGeneral/SFXCallback/SFXTimer.h.hpp>

class SFYSingleTextWidget : public SFYWidget {
    SFMSEALRESPONDER(SFYSingleTextWidget)
    SFMRESPONDERINSTANTIATETWO(SFYSingleTextWidget, SFYWidget, SFYResponder)
    public:
        enum CodeEnum {
            CODE_TYPE                           = four_char_code('+', 's', 't', 'x')
        };
        SFMTYPEDEFTYPE(CodeEnum)
        enum DefaultEnum {
            DEFAULT_WAIT                        = 1000,
            DEFAULT_SCROLL                      = 100,
            DEFAULT_STEP                        = -1
        };
        SFMTYPEDEFTYPE(DefaultEnum)
        enum HorizontalEnum {
            HORIZONTAL_LEFT                     = 0,
            HORIZONTAL_CENTER,
            HORIZONTAL_RIGHT,
            DEFAULT_HORIZONTAL                  = HORIZONTAL_CENTER
        };
        SFMTYPEDEFTYPE(HorizontalEnum)
        enum VerticalEnum {
            VERTICAL_TOP                        = 0,
            VERTICAL_MIDDLE,
            VERTICAL_BOTTOM,
            DEFAULT_VERTICAL                    = VERTICAL_MIDDLE
        };
        SFMTYPEDEFTYPE(VerticalEnum)
        enum DirectionEnum {
            DIRECTION_LEFT                      = 0,
            DIRECTION_RIGHT,
            DEFAULT_DIRECTION                   = DIRECTION_LEFT
        };
        SFMTYPEDEFTYPE(DirectionEnum)
    private:
        enum StateEnum {
            STATE_IDLE                          = 0,
            STATE_WAIT,
            STATE_SCROLL
        };
        SFMTYPEDEFTYPE(StateEnum)

    private:
        SFMTYPEDEFSTRUCT(ColorRec)
        struct ColorRec {
            SFXRGBColor                         text;
        };
        SFMTYPEDEFSTRUCT(AlignRec)
        struct AlignRec {
            HorizontalEnum                      horizontal;
            VerticalEnum                        vertical;
        };

    private:
                StateEnum                       _state;
                SFXWideString                   _text;
                AEEFont                         _font;
                ColorRec                        _color;
                AlignRec                        _align;
                SFXRectangle                    _bound;
                SFXSize                         _size;
                SFXTimer                        _timer;
                UInt32                          _wait;
                UInt32                          _scroll;
                DirectionEnum                   _direction;
                SInt16                          _step;
                SInt16                          _offset;

    public:
        static  SFYSingleTextWidgetSmp          NewInstance                     (SFCErrorPtr exception = null);
                SFCError                        SetText                         (SFXPathConstRef path, UInt16 id);
                SFCError                        SetText                         (SFXWideStringConstRef param);
                SFXWideStringConstRef           GetText                         (Void) const;
                SFXSizeConstRef                 GetTextSize                     (Void) const;
                Void                            SetTextColor                    (SFXRGBColorConstRef param);
                SFXRGBColorConstRef             GetTextColor                    (Void) const;
                Void                            SetFont                         (AEEFont param);
                AEEFont                         GetFont                         (Void) const;
                SFXSizeConstRef                 GetFontSize                     (Void) const;
                Void                            SetHorizontalAlign              (HorizontalEnum param);
                HorizontalEnum                  GetHorizontalAlign              (Void) const;
                Void                            SetVerticalAlign                (VerticalEnum param);
                VerticalEnum                    GetVerticalAlign                (Void) const;
                Void                            SetWaitInterval                 (UInt32 param);
                UInt32                          GetWaitInterval                 (Void) const;
                Void                            SetScrollInterval               (UInt32 param);
                UInt32                          GetScrollInterval               (Void) const;
                Void                            SetScrollDirection              (DirectionEnum param);
                DirectionEnum                   GetScrollDirection              (Void) const;
                Void                            SetScrollStep                   (SInt16 param);
                SInt16                          GetScrollStep                   (Void) const;
                Void                            StartScroll                     (Void);
                Void                            StopScroll                      (Void);
    protected:
        explicit                                SFYSingleTextWidget             (Void) static_throws;
        virtual                                 ~SFYSingleTextWidget            (Void);
                SFXGridConstRef                 GetTextOrigin                   (Void) const;
                SInt16                          GetScrollOffset                 (Void) const;
        virtual Void                            HandleBoundRequest              (SFXRectanglePtr rectangle) const;
        virtual Void                            HandleBoundOptimize             (SFXRectanglePtr rectangle) const;
        virtual Void                            HandleBoundReal                 (Void);
        virtual Void                            HandleBoundVirtual              (Void);
        virtual Void                            HandleRenderRequest             (SFXGraphicsPtr graphics) const;
                Void                            DrawText                        (SFXGraphicsPtr graphics, SFXGridConstRef grid) const;
    private:
                XANDLER_DECLARE_VOIDRESUME(OnAppResume)
                XANDLER_DECLARE_VOIDSUSPEND(OnAppSuspend)
                XANDLER_DECLARE_VOIDSTATE(OnStateValid)
                XALLBACK_DECLARE_SFXTIMER(OnTimer)
                Bool                            Scroll                          (SInt16 param);
                SInt16                          Measure                         (Void);
                Void                            Relocate                        (Void);
};

#include <SFYResponder/SFYWidget/SFYSingleTextWidget.i.hpp>

#endif // __SOPHIAFRAMEWORK_SFYSINGLETEXTWIDGET_HHPP //
