/**
***     Copyright (C) 2008 - 2010 Sophia Cradle Incorporated.
***     All rights reserved.
***
***     This source code was written by HORIGUCHI Junshi.
***
***     Project         : SophiaFramework
***     Codename        : Graviton
***     Platform        : x86, ARM
***     OS              : BREW 2.0, BREW 2.1, BREW 3.1, BREW 4.0
***     Compiler        : Visual C++ 6.0, Visual C++ .NET 2003, Visual C++ 2005,
***                       ADS 1.2, RVCTB 1.2, RVCTB 3.0,
***                       GNUARM 4.1.1, YAGARTO 4.4.2
***     Package         : Graphical User Interface Package
***     File            : SFZSingleEditLabelControl.h.hpp
**/

#ifndef __SOPHIAFRAMEWORK_SFZSINGLEEDITLABELCONTROL_HHPP
#define __SOPHIAFRAMEWORK_SFZSINGLEEDITLABELCONTROL_HHPP

#include <SFYResponder/SFYEnvironment.h.hpp>
#include <SFZResponder/SFZControl/SFZLabelControl/SFZSingleEditLabelControl.f.hpp>
#include <SFYResponder/SFYControl/SFYLabelControl/SFYLabelControl.h.hpp>
#include <SFYResponder/SFYWidget/SFYSingleEditWidget.h.hpp>

class SFZSingleEditLabelControl : public SFYLabelControl {
    SFMSEALRESPONDER(SFZSingleEditLabelControl)
    SFMRESPONDERINSTANTIATEFOUR(SFZSingleEditLabelControl, SFYLabelControl, SFYControl, SFYWidget, SFYResponder)
    public:
        enum CodeEnum {
            CODE_TYPE                           = four_char_code('c', 's', 'e', 'l')
        };
        SFMTYPEDEFTYPE(CodeEnum)
        enum DefaultEnum {
            DEFAULT_WAIT                        = SFYSingleEditWidget::DEFAULT_WAIT,
            DEFAULT_SCROLL                      = SFYSingleEditWidget::DEFAULT_SCROLL,
            DEFAULT_STEP                        = SFYSingleEditWidget::DEFAULT_STEP
        };
        SFMTYPEDEFTYPE(DefaultEnum)
        enum HorizontalEnum {
            HORIZONTAL_LEFT                     = SFYSingleEditWidget::HORIZONTAL_LEFT,
            HORIZONTAL_CENTER                   = SFYSingleEditWidget::HORIZONTAL_CENTER,
            HORIZONTAL_RIGHT                    = SFYSingleEditWidget::HORIZONTAL_RIGHT,
            DEFAULT_HORIZONTAL                  = SFYSingleEditWidget::DEFAULT_HORIZONTAL
        };
        SFMTYPEDEFTYPE(HorizontalEnum)
        enum VerticalEnum {
            VERTICAL_TOP                        = SFYSingleEditWidget::VERTICAL_TOP,
            VERTICAL_MIDDLE                     = SFYSingleEditWidget::VERTICAL_MIDDLE,
            VERTICAL_BOTTOM                     = SFYSingleEditWidget::VERTICAL_BOTTOM,
            DEFAULT_VERTICAL                    = SFYSingleEditWidget::DEFAULT_VERTICAL
        };
        SFMTYPEDEFTYPE(VerticalEnum)
        enum DirectionEnum {
            DIRECTION_LEFT                      = SFYSingleEditWidget::DIRECTION_LEFT,
            DIRECTION_RIGHT                     = SFYSingleEditWidget::DIRECTION_RIGHT,
            DEFAULT_DIRECTION                   = SFYSingleEditWidget::DEFAULT_DIRECTION
        };
        SFMTYPEDEFTYPE(DirectionEnum)

    private:
                SFYSingleEditWidgetSmp          _widget;

    public:
        static  SFZSingleEditLabelControlSmp    NewInstance                     (SFCErrorPtr exception = null);
                SFCError                        SetText                         (SFXPathConstRef path, UInt16 id);
                SFCError                        SetText                         (SFXWideStringConstRef param);
                SFXWideStringConstRef           GetText                         (Void) const;
                Void                            SetTextColor                    (SFXRGBColorConstRef param);
                SFXRGBColorConstRef             GetTextColor                    (Void) const;
                Void                            SetFont                         (AEEFont param);
                AEEFont                         GetFont                         (Void) const;
                Void                            SetHorizontalAlign              (HorizontalEnum param);
                HorizontalEnum                  GetHorizontalAlign              (Void) const;
                Void                            SetVerticalAlign                (VerticalEnum param);
                VerticalEnum                    GetVerticalAlign                (Void) const;
                SFCError                        SetMaximumLength                (UInt16 param);
                UInt16                          GetMaximumLength                (Void) const;
                Void                            SetInputMode                    (AEETextInputMode param);
                AEETextInputMode                GetInputMode                    (Void) const;
                SFCError                        SetPasswordMode                 (Bool param);
                Bool                            GetPasswordMode                 (Void) const;
                Void                            SetWaitInterval                 (UInt32 param);
                UInt32                          GetWaitInterval                 (Void) const;
                Void                            SetScrollInterval               (UInt32 param);
                UInt32                          GetScrollInterval               (Void) const;
                Void                            SetScrollDirection              (DirectionEnum param);
                DirectionEnum                   GetScrollDirection              (Void) const;
                Void                            SetScrollStep                   (SInt16 param);
                SInt16                          GetScrollStep                   (Void) const;
                Void                            StartScroll                     (Void);
                Void                            StopScroll                      (Void);
    protected:
        explicit                                SFZSingleEditLabelControl       (Void) static_throws;
        virtual                                 ~SFZSingleEditLabelControl      (Void);
                SFYSingleEditWidgetSmpConstRef  GetWidget                       (Void) const;
        virtual Void                            HandleOperateKey                (Void);
        virtual Void                            HandleBoundRequest              (SFXRectanglePtr rectangle) const;
        virtual Void                            HandleBoundOptimize             (SFXRectanglePtr rectangle) const;
        virtual Void                            HandleBoundVirtual              (Void);
        virtual Void                            HandleRenderRequest             (SFXGraphicsPtr graphics) const;
    private:
                XANDLER_DECLARE_VOIDSTATE(OnStateFocus)
                XANDLER_DECLARE_VOIDSTYLE(OnWidgetStyle)
                XANDLER_DECLARE_VOIDRESULT(OnWidgetResult)
                Void                            Relocate                        (Void);
//////////////////////////////////////// Will be deprecated in version 6.0 //////////
    public:
                Void                            SetSFBTextCtlClassID            (AEECLSID param);
                AEECLSID                        GetSFBTextCtlClassID            (Void) const;
                Void                            SetSFBMenuCtlClassID            (AEECLSID param);
                AEECLSID                        GetSFBMenuCtlClassID            (Void) const;
/////////////////////////////////////////////////////////////////////////////////////
};

#include <SFZResponder/SFZControl/SFZLabelControl/SFZSingleEditLabelControl.i.hpp>

#endif // __SOPHIAFRAMEWORK_SFZSINGLEEDITLABELCONTROL_HHPP //
