/**
***     Copyright (C) 2008 - 2010 Sophia Cradle Incorporated.
***     All rights reserved.
***
***     This source code was written by HORIGUCHI Junshi.
***
***     Project         : SophiaFramework
***     Codename        : Graviton
***     Platform        : x86, ARM
***     OS              : BREW 2.0, BREW 2.1, BREW 3.1, BREW 4.0
***     Compiler        : Visual C++ 6.0, Visual C++ .NET 2003, Visual C++ 2005,
***                       ADS 1.2, RVCTB 1.2, RVCTB 3.0,
***                       GNUARM 4.1.1, YAGARTO 4.4.2
***     Package         : Graphical User Interface Package
***     File            : SFYSingleEditWidget.i.cpp
**/

#include <SFYResponder/SFYWidget/SFYSingleEditWidget.h.hpp>
#if defined TARGET_LANGUAGE_JAPANESE
#include <SFXString/SFXShiftJIS.h.hpp>
#endif
#include <SFXStorage/SFXFile/SFXPath.h.hpp>
#include <SFYResponder/SFYApplication/SFYApplication.h.hpp>

#if !defined TARGET_LANGUAGE_JAPANESE
#define     PASSWORD_ASCII                                                      ('*')
#endif

/*protected */SFYSingleEditWidget::SFYSingleEditWidget(Void) static_throws
{
    static SFXEventRange::AtomRecConst          crange[] = {
        { SFEVT_RESPONDER_STYLE,  SFEVT_RESPONDER_STYLE,          SFP16_BEGIN,            SFP16_END}
    };
    SFYHandler::RuleRec                         crule[lengthof(crange)];
    SFCError                                    error;

    if (static_try()) {
        if ((_editor = SFYApplication::GetEditor()) != null) {
            SetType(CODE_TYPE);
            if ((_widget = SFYSingleTextWidget::NewInstance(&error)) != null) {
                crule[0].spp = XANDLER_FUNCTION(OnWidgetStyle);
                crule[0].reference = this;
                static_throw(_widget->RegisterHandler(atomic_cast(crange), crule, lengthof(crange)));
                if (static_try()) {
                    static_throw(_widget->SetParent(GetThis()));
                    if (static_try()) {
                        static_throw(_text.Set(_property.GetText()));
                        if (static_try()) {
                            static_throw(SetWidgetText(_text, _property.GetPasswordMode()));
                            if (static_try()) {
                                SetPropertyTransparent(true);
                                _widget->SetState(true, true, false, false);
                                _maximum = _property.GetMaximumLength();
                                Relocate();
                            }
                        }
                    }
                }
            }
            else {
                static_throw(error);
            }
        }
        else {
            static_throw(SFERR_INVALID_STATE);
        }
    }
}// SFYSingleEditWidget::SFYSingleEditWidget //

/*protected virtual */SFYSingleEditWidget::~SFYSingleEditWidget(Void)
{
}// SFYSingleEditWidget::~SFYSingleEditWidget //

/*public static */SFYSingleEditWidgetSmp SFYSingleEditWidget::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<SFYSingleEditWidget>(Factory(::new SFYSingleEditWidget, exception));
}// SFYSingleEditWidget::NewInstance //

/*public */SFCError SFYSingleEditWidget::SetText(SFXPathConstRef path, UInt16 id)
{
    SFBShellSmp                                 shell;
    SFCError                                    error(SFERR_NO_ERROR);

    if ((shell = SFBShell::GetInstance()) != null) {
        error = SetText(shell->LoadResString(path.Get(), id));
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}// SFYSingleEditWidget::SetText //

/*public */SFCError SFYSingleEditWidget::SetText(SFXWideStringConstRef param)
{
    return SetTextMaximumLength(param, _maximum);
}// SFYSingleEditWidget::SetText //

/*public */SFCError SFYSingleEditWidget::SetMaximumLength(UInt16 param)
{
    return SetTextMaximumLength(_text, param);
}// SFYSingleEditWidget::SetMaximumLength //

/*public */Void SFYSingleEditWidget::SetInputMode(AEETextInputMode param)
{
    if (param != _property.GetInputMode()) {
        _property.SetInputMode(param);
        Invalidate();
    }
    return;
}// SFYSingleEditWidget::SetInputMode //

/*public */SFCError SFYSingleEditWidget::SetPasswordMode(Bool param)
{
    SFCError                                    error(SFERR_NO_ERROR);

    if (param != _property.GetPasswordMode()) {
        if ((error = SetWidgetText(_text, param)) == SFERR_NO_ERROR) {
            _property.SetPasswordMode(param);
            Invalidate();
        }
    }
    return error;
}// SFYSingleEditWidget::SetPasswordMode //

/*public */SFCError SFYSingleEditWidget::Edit(Void)
{
    SFCError                                    error(SFERR_NO_ERROR);

    if ((error = _property.SetMaximumLength(_maximum)) == SFERR_NO_ERROR) {
        if ((error = _property.SetText(_text)) == SFERR_NO_ERROR) {
            error = _editor->Open(&_property, XALLBACK_INTERNAL(OnEditor));
        }
    }
    return error;
}// SFYSingleEditWidget::Edit //

/*protected virtual */Void SFYSingleEditWidget::HandleBoundRequest(SFXRectanglePtr rectangle) const
{
    rectangle->SetSize(_widget->GetSuitableBound().GetSize());
    return;
}// SFYSingleEditWidget::HandleBoundRequest //

/*protected virtual */Void SFYSingleEditWidget::HandleBoundOptimize(SFXRectanglePtr rectangle) const
{
    rectangle->SetSize(_widget->GetSuitableBound(*rectangle).GetSize());
    return;
}// SFYSingleEditWidget::HandleBoundOptimize //

/*protected virtual */Void SFYSingleEditWidget::HandleBoundReal(Void)
{
    SetVirtualBound(SFXRectangle(SFXGrid::ZeroInstance(), GetRealBound().GetSize()));
    return;
}// SFYSingleEditWidget::HandleBoundReal //

/*protected virtual */Void SFYSingleEditWidget::HandleBoundVirtual(Void)
{
    Relocate();
    return;
}// SFYSingleEditWidget::HandleBoundVirtual //

/*private */XANDLER_IMPLEMENT_VOIDSTYLE(SFYSingleEditWidget, OnWidgetStyle, invoker, reason)
{
    unused(invoker);
    switch (reason) {
        case SFP16_STYLE_TEXT:
            break;
        default:
            InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, reason, 0), true);
            break;
    }
    return;
}// XANDLER_IMPLEMENT_VOIDSTYLE(SFYSingleEditWidget, OnWidgetStyle) //

/*private */XALLBACK_IMPLEMENT_SFXEDITOR(SFYSingleEditWidget, OnEditor, context)
{
    SFYResponderSmp                             root;
    Bool                                        handle;
    SFCError                                    error;

    error = SFERR_NO_ERROR;
    if ((root = GetRoot()) != null) {
        handle = false;
        if ((error = root->Distribute(reinterpret_cast<VoidPtr>(context), XALLBACK_INTERNAL(OnDistribute), &handle)) == SFERR_NO_ERROR) {
            if (handle) {
                error = root->Render(true);
            }
        }
    }
    else {
        error = SFERR_FAILED;
    }
    if (error != SFERR_NO_ERROR) {
        // internal fault!
    }
    return;
}// XALLBACK_IMPLEMENT_SFXEDITOR(SFYSingleEditWidget, OnEditor) //

/*private */XALLBACK_IMPLEMENT_SFYDISTRIBUTER(SFYSingleEditWidget, OnDistribute, context)
{
    SFCError                                    error;

    if ((error = reinterpret_cast<SFCError>(context)) == SFERR_NO_ERROR) {
        if ((error = SetTextMaximumLength(_property.GetText(), _property.GetMaximumLength())) == SFERR_NO_ERROR) {
            InvokeForward(SFXEvent(SFEVT_RESPONDER_RESULT, SFP16_RESULT_OK, 0), false);
        }
    }
    if (error != SFERR_NO_ERROR) {
        InvokeForward(SFXEvent(SFEVT_RESPONDER_RESULT, SFP16_RESULT_ERROR, error), false);
    }
    return true;
}// XALLBACK_IMPLEMENT_SFYDISTRIBUTER(SFYSingleEditWidget, OnDistribute) //

/*private */SFCError SFYSingleEditWidget::SetTextMaximumLength(SFXWideStringConstRef text, UInt16 maximum)
{
    SFXWideString                               string;
    SFXWideString                               temp;
    SFCError                                    error(SFERR_NO_ERROR);

    if (!text.Equals(_text) || maximum != _maximum) {
        if ((error = string.Set(text.Substring(0, maximum))) == SFERR_NO_ERROR) {
            if (!string.Equals(_text)) {
                if ((error = temp.Set(_text)) == SFERR_NO_ERROR) {
                    if ((error = _text.Set(string)) == SFERR_NO_ERROR) {
                        if ((error = SetWidgetText(_text, _property.GetPasswordMode())) == SFERR_NO_ERROR) {
                            _maximum = maximum;
                            Invalidate();
                            InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, SFP16_STYLE_TEXT, 0), true);
                        }
                        else if (temp.IsEmpty()) {
                            _text.Clear();
                        }
                        else if (_text.Attach(&temp) != SFERR_NO_ERROR) {
                            _text.Clear();
                            Invalidate();
                            InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, SFP16_STYLE_TEXT, 0), true);
                        }
                    }
                }
            }
            else if (maximum != _maximum) {
                _maximum = maximum;
                Invalidate();
            }
        }
    }
    return error;
}// SFYSingleEditWidget::SetTextMaximumLength //

/*private */SFCError SFYSingleEditWidget::SetWidgetText(SFXWideStringConstRef text, Bool password)
{
    SFXWideString                               string;
    SFCError                                    error(SFERR_NO_ERROR);

    if (password) {
        if ((error = string.Set(text)) == SFERR_NO_ERROR) {
            #if defined TARGET_LANGUAGE_JAPANESE
            error = SFXShiftJIS::ToPassword(&string);
            #else
            string.Fill(PASSWORD_ASCII);
            #endif
            if (error == SFERR_NO_ERROR) {
                error = _widget->SetText(string);
            }
        }
    }
    else {
        error = _widget->SetText(text);
    }
    return error;
}// SFYSingleEditWidget::SetWidgetText //

/*private */Void SFYSingleEditWidget::Relocate(Void)
{
    _widget->SetRealBound(GetLocalBound());
    return;
}// SFYSingleEditWidget::Relocate //

//////////////////////////////////////// Will be deprecated in version 6.0 //////////
/*public */Void SFYSingleEditWidget::SetSFBTextCtlClassID(AEECLSID param)
{
    if (param != _editor->GetSFBTextCtlClassID()) {
        _editor->SetSFBTextCtlClassID(param);
        Invalidate();
    }
    return;
}// SFYSingleEditWidget::SetSFBTextCtlClassID //

/*public */Void SFYSingleEditWidget::SetSFBMenuCtlClassID(AEECLSID param)
{
    if (param != _editor->GetSFBMenuCtlClassID()) {
        _editor->SetSFBMenuCtlClassID(param);
        Invalidate();
    }
    return;
}// SFYSingleEditWidget::SetSFBMenuCtlClassID //
/////////////////////////////////////////////////////////////////////////////////////

