/**
***     Copyright (C) 2007 - 2010 Sophia Cradle Incorporated.
***     All rights reserved.
***
***     This source code was written by HORIGUCHI Junshi.
***
***     Project         : SophiaFramework
***     Codename        : Graviton
***     Platform        : x86, ARM
***     OS              : BREW 2.0, BREW 2.1, BREW 3.1, BREW 4.0
***     Compiler        : Visual C++ 6.0, Visual C++ .NET 2003, Visual C++ 2005,
***                       ADS 1.2, RVCTB 1.2, RVCTB 3.0,
***                       GNUARM 4.1.1, YAGARTO 4.4.2
***     Package         : Graphical User Interface Package
***     File            : SFYSingleTextWidget.i.cpp
**/

#include <SFYResponder/SFYWidget/SFYSingleTextWidget.h.hpp>
#include <SFBWrapper/SFBShell.h.hpp>
#include <SFXGraphics/SFXGraphics.h.hpp>
#include <SFXStorage/SFXFile/SFXPath.h.hpp>

/*protected */SFYSingleTextWidget::SFYSingleTextWidget(Void) static_throws : _state(STATE_IDLE)
{
    static SFXRGBColor::AtomRecConst            rgb[] = {
        {{{0x00, 0x00, 0x00, 0x00}}}
    };
    static SFXEventRange::AtomRecConst          trange[] = {
        {      SFEVT_APP_RESUME,       SFEVT_APP_RESUME,          SFP16_BEGIN,            SFP16_END},
        {     SFEVT_APP_SUSPEND,      SFEVT_APP_SUSPEND,          SFP16_BEGIN,            SFP16_END},
        { SFEVT_RESPONDER_STATE,  SFEVT_RESPONDER_STATE,    SFP16_STATE_VALID,    SFP16_STATE_VALID}
    };
    SFYHandler::RuleRec                         trule[lengthof(trange)];

    if (static_try()) {
        SetType(CODE_TYPE);
        trule[0].spp = XANDLER_FUNCTION(OnAppResume);
        trule[0].reference = this;
        trule[1].spp = XANDLER_FUNCTION(OnAppSuspend);
        trule[1].reference = this;
        trule[2].spp = XANDLER_FUNCTION(OnStateValid);
        trule[2].reference = this;
        static_throw(RegisterHandler(atomic_cast(trange), trule, lengthof(trange)));
        if (static_try()) {
            SetPropertyTransparent(true);
            _font = AEE_FONT_NORMAL;
            _color.text.Set(rgb[0]);
            _align.horizontal = DEFAULT_HORIZONTAL;
            _align.vertical = DEFAULT_VERTICAL;
            _timer.Set(XALLBACK_INTERNAL(OnTimer));
            _wait = DEFAULT_WAIT;
            _scroll = DEFAULT_SCROLL;
            _direction = DEFAULT_DIRECTION;
            _step = Measure();
            Relocate();
        }
    }
}// SFYSingleTextWidget::SFYSingleTextWidget //

/*protected virtual */SFYSingleTextWidget::~SFYSingleTextWidget(Void)
{
    StopScroll();
}// SFYSingleTextWidget::~SFYSingleTextWidget //

/*public static */SFYSingleTextWidgetSmp SFYSingleTextWidget::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<SFYSingleTextWidget>(Factory(::new SFYSingleTextWidget, exception));
}// SFYSingleTextWidget::NewInstance //

/*public */SFCError SFYSingleTextWidget::SetText(SFXPathConstRef path, UInt16 id)
{
    SFBShellSmp                                 shell;
    SFCError                                    error(SFERR_NO_ERROR);

    if ((shell = SFBShell::GetInstance()) != null) {
        error = SetText(shell->LoadResString(path.Get(), id));
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}// SFYSingleTextWidget::SetText //

/*public */SFCError SFYSingleTextWidget::SetText(SFXWideStringConstRef param)
{
    StateEnum                                   state;
    SFCError                                    error(SFERR_NO_ERROR);

    if (!param.Equals(_text)) {
        if ((error = _text.Set(param)) == SFERR_NO_ERROR) {
            state = _state;
            StopScroll();
            Measure();
            Relocate();
            Invalidate();
            if (state != STATE_IDLE) {
                StartScroll();
            }
            InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, SFP16_STYLE_TEXT, 0), true);
        }
    }
    return error;
}// SFYSingleTextWidget::SetText //

/*public */Void SFYSingleTextWidget::SetTextColor(SFXRGBColorConstRef param)
{
    if (!param.Equals(_color.text)) {
        _color.text.Set(param);
        Invalidate();
        InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, SFP16_STYLE_COLOR, 0), true);
    }
    return;
}// SFYSingleTextWidget::SetTextColor //

/*public */Void SFYSingleTextWidget::SetFont(AEEFont param)
{
    StateEnum                                   state;

    if (param != _font) {
        _font = param;
        state = _state;
        StopScroll();
        _step = Measure();
        Relocate();
        Invalidate();
        if (state != STATE_IDLE) {
            StartScroll();
        }
        InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, SFP16_STYLE_FONT, 0), true);
    }
    return;
}// SFYSingleTextWidget::SetFont //

/*public */Void SFYSingleTextWidget::SetHorizontalAlign(HorizontalEnum param)
{
    if (param != _align.horizontal) {
        _align.horizontal = param;
        Relocate();
        Invalidate();
        InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, SFP16_STYLE_ALIGN, 0), true);
    }
    return;
}// SFYSingleTextWidget::SetHorizontalAlign //

/*public */Void SFYSingleTextWidget::SetVerticalAlign(VerticalEnum param)
{
    if (param != _align.vertical) {
        _align.vertical = param;
        Relocate();
        Invalidate();
        InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, SFP16_STYLE_ALIGN, 0), true);
    }
    return;
}// SFYSingleTextWidget::SetVerticalAlign //

/*public */Void SFYSingleTextWidget::SetWaitInterval(UInt32 param)
{
    if (param != _wait) {
        _wait = param;
        Invalidate();
    }
    return;
}// SFYSingleTextWidget::SetWaitInterval //

/*public */Void SFYSingleTextWidget::SetScrollInterval(UInt32 param)
{
    if (param != _scroll) {
        _scroll = param;
        Invalidate();
    }
    return;
}// SFYSingleTextWidget::SetScrollInterval //

/*public */Void SFYSingleTextWidget::SetScrollDirection(DirectionEnum param)
{
    if (param != _direction) {
        _direction = param;
        Relocate();
        Invalidate();
    }
    return;
}// SFYSingleTextWidget::SetScrollDirection //

/*public */Void SFYSingleTextWidget::SetScrollStep(SInt16 param)
{
    if (param < 0) {
        param = _size.GetWidth();
    }
    if (param != _step) {
        _step = param;
        Invalidate();
    }
    return;
}// SFYSingleTextWidget::SetScrollStep //

/*public */Void SFYSingleTextWidget::StartScroll(Void)
{
    StopScroll();
    if (_state == STATE_IDLE) {
        if (_bound.GetWidth() > GetLocalBound().GetWidth()) {
            _state = STATE_WAIT;
            Relocate();
            Invalidate();
            _timer.Schedule(_wait);
        }
    }
    return;
}// SFYSingleTextWidget::StartScroll //

/*public */Void SFYSingleTextWidget::StopScroll(Void)
{
    if (_state != STATE_IDLE) {
        _timer.Cancel();
        Relocate();
        Invalidate();
        _state = STATE_IDLE;
    }
    return;
}// SFYSingleTextWidget::StopScroll //

/*protected virtual */Void SFYSingleTextWidget::HandleBoundRequest(SFXRectanglePtr rectangle) const
{
    rectangle->SetSize(_bound.GetSize());
    return;
}// SFYSingleTextWidget::HandleBoundRequest //

/*protected virtual */Void SFYSingleTextWidget::HandleBoundOptimize(SFXRectanglePtr rectangle) const
{
    if (rectangle->GetWidth() > _bound.GetWidth()) {
        rectangle->SetWidth(_bound.GetWidth());
    }
    else if (rectangle->GetWidth() > _size.GetWidth()) {
        rectangle->DivWidth(_size.GetWidth());
        rectangle->MulWidth(_size.GetWidth());
    }
    if (rectangle->GetHeight() > _bound.GetHeight()) {
        rectangle->SetHeight(_bound.GetHeight());
    }
    else if (rectangle->GetHeight() > _size.GetHeight()) {
        rectangle->DivHeight(_size.GetHeight());
        rectangle->MulHeight(_size.GetHeight());
    }
    return;
}// SFYSingleTextWidget::HandleBoundOptimize //

/*protected virtual */Void SFYSingleTextWidget::HandleBoundReal(Void)
{
    SetVirtualBound(SFXRectangle(SFXGrid::ZeroInstance(), GetRealBound().GetSize()));
    return;
}// SFYSingleTextWidget::HandleBoundReal //

/*protected virtual */Void SFYSingleTextWidget::HandleBoundVirtual(Void)
{
    StateEnum                                   state;

    state = _state;
    StopScroll();
    Relocate();
    if (state != STATE_IDLE) {
        StartScroll();
    }
    return;
}// SFYSingleTextWidget::HandleBoundVirtual //

/*protected virtual */Void SFYSingleTextWidget::HandleRenderRequest(SFXGraphicsPtr graphics) const
{
    DrawText(graphics, _bound.GetOrigin());
    return;
}// SFYSingleTextWidget::HandleRenderRequest //

/*protected */Void SFYSingleTextWidget::DrawText(SFXGraphicsPtr graphics, SFXGridConstRef grid) const
{
    SFXGrid                                     origin;
    SFXRGBColor                                 rgb;

    origin.Set(grid);
    origin.AddX(_offset);
    rgb.Set(_color.text);
    if (!GetStateActive(true)) {
        if (rgb.GetBrightness() > 0x7F) {
            rgb.SubRGB(0x44);
        }
        else {
            rgb.AddRGB(0x44);
        }
    }
    graphics->SetFont(_font);
    graphics->DrawSingleText(_text, origin, rgb);
    return;
}// SFYSingleTextWidget::DrawText //

/*private */XANDLER_IMPLEMENT_VOIDRESUME(SFYSingleTextWidget, OnAppResume, invoker, environment)
{
    unused(invoker);
    unused(environment);
    switch (_state) {
        case STATE_WAIT:
            _timer.Schedule(_wait);
            break;
        case STATE_SCROLL:
            _timer.Schedule(_scroll);
            break;
        case STATE_IDLE:
        default:
            break;
    }
    return;
}// XANDLER_IMPLEMENT_VOIDRESUME(SFYSingleTextWidget, OnAppResume) //

/*private */XANDLER_IMPLEMENT_VOIDSUSPEND(SFYSingleTextWidget, OnAppSuspend, invoker, reason, info)
{
    unused(invoker);
    unused(reason);
    unused(info);
    if (_state != STATE_IDLE) {
        _timer.Cancel();
    }
    return;
}// XANDLER_IMPLEMENT_VOIDSUSPEND(SFYSingleTextWidget, OnAppSuspend) //

/*private */XANDLER_IMPLEMENT_VOIDSTATE(SFYSingleTextWidget, OnStateValid, invoker, reason, state)
{
    unused(invoker);
    unused(reason);
    if (!state) {
        StopScroll();
    }
    return;
}// XANDLER_IMPLEMENT_VOIDSTATE(SFYSingleTextWidget, OnStateValid) //

/*private */XALLBACK_IMPLEMENT_SFXTIMER(SFYSingleTextWidget, OnTimer)
{
    SInt16                                      offset;

    switch (_state) {
        case STATE_WAIT:
            _state = STATE_SCROLL;
        case STATE_SCROLL:
            offset = 0;
            switch (_direction) {
                case DIRECTION_RIGHT:
                    offset = +_step;
                    break;
                case DIRECTION_LEFT:
                default:
                    offset = -_step;
                    break;
            }
            if (Scroll(offset)) {
                Render();
            }
            _timer.Schedule(_scroll);
            break;
        case STATE_IDLE:
        default:
            break;
    }
    return;
}// XALLBACK_IMPLEMENT_SFXTIMER(SFYSingleTextWidget, OnTimer) //

/*private */Bool SFYSingleTextWidget::Scroll(SInt16 param)
{
    SFXRectangle                                lx;
    Bool                                        result(false);

    lx.Set(GetLocalBound());
    if ((param += _offset) < -_bound.GetWidth()) {
        param = lx.GetWidth();
    }
    if (param > lx.GetWidth()) {
        param = -_bound.GetWidth();
    }
    if (param != _offset) {
        _offset = param;
        Invalidate();
        result = true;
    }
    return result;
}// SFYSingleTextWidget::Scroll //

/*private */SInt16 SFYSingleTextWidget::Measure(Void)
{
    _bound.SetSize(SFXSize::ZeroInstance());
    _size.Set(SFXGraphics::MeasureSingleText(_font, " "), SFXGraphics::GetFontHeight(_font));
    if (!_text.IsEmpty()) {
        _bound.SetSize(SFXGraphics::MeasureSingleText(_font, _text), _size.GetHeight());
    }
    return _size.GetWidth();
}// SFYSingleTextWidget::Measure //

/*private */Void SFYSingleTextWidget::Relocate(Void)
{
    SFXRectangle                                lx;

    lx.Set(GetLocalBound());
    _bound.SetOrigin(lx.GetOrigin());
    _offset = 0;
    if (lx.GetWidth() > _bound.GetWidth()) {
        switch (_align.horizontal) {
            case HORIZONTAL_LEFT:
                _bound.AddX(0);
                break;
            case HORIZONTAL_RIGHT:
                _bound.AddX(lx.GetWidth() - _bound.GetWidth());
                break;
            case HORIZONTAL_CENTER:
            default:
                _bound.AddX((lx.GetWidth() - _bound.GetWidth()) / 2);
                break;
        }
    }
    else {
        switch (_align.horizontal) {
            case HORIZONTAL_LEFT:
                _offset += 0;
                break;
            case HORIZONTAL_RIGHT:
                _offset += lx.GetWidth() - _bound.GetWidth();
                break;
            case HORIZONTAL_CENTER:
            default:
                _offset += (lx.GetWidth() - _bound.GetWidth()) / 2;
                break;
        }
    }
    switch (_align.vertical) {
        case VERTICAL_TOP:
            _bound.AddY(0);
            break;
        case VERTICAL_BOTTOM:
            _bound.AddY(lx.GetHeight() - _bound.GetHeight());
            break;
        case VERTICAL_MIDDLE:
        default:
            _bound.AddY((lx.GetHeight() - _bound.GetHeight()) / 2);
            break;
    }
    _bound.AddY(SFXGraphics::GetFontDescent(_font) / 2);
    return;
}// SFYSingleTextWidget::Relocate //
