/**
***     Copyright (C) 2008 - 2010 Sophia Cradle Incorporated.
***     All rights reserved.
***
***     This source code was written by HORIGUCHI Junshi.
***
***     Project         : SophiaFramework
***     Codename        : Graviton
***     Platform        : x86, ARM
***     OS              : BREW 2.0, BREW 2.1, BREW 3.1, BREW 4.0
***     Compiler        : Visual C++ 6.0, Visual C++ .NET 2003, Visual C++ 2005,
***                       ADS 1.2, RVCTB 1.2, RVCTB 3.0,
***                       GNUARM 4.1.1, YAGARTO 4.4.2
***     Package         : Graphical User Interface Package
***     File            : SFZSingleEditLabelControl.i.cpp
**/

#include <SFZResponder/SFZControl/SFZLabelControl/SFZSingleEditLabelControl.h.hpp>

/*protected */SFZSingleEditLabelControl::SFZSingleEditLabelControl(Void) static_throws
{
    static SFXEventRange::AtomRecConst          trange[] = {
        { SFEVT_RESPONDER_STATE,  SFEVT_RESPONDER_STATE,    SFP16_STATE_FOCUS,    SFP16_STATE_FOCUS}
    };
    static SFXEventRange::AtomRecConst          crange[] = {
        { SFEVT_RESPONDER_STYLE,  SFEVT_RESPONDER_STYLE,          SFP16_BEGIN,            SFP16_END},
        {SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT,          SFP16_BEGIN,            SFP16_END}
    };
    SFYHandler::RuleRec                         trule[lengthof(trange)];
    SFYHandler::RuleRec                         crule[lengthof(crange)];
    SFCError                                    error;

    if (static_try()) {
        SetType(CODE_TYPE);
        trule[0].spp = XANDLER_FUNCTION(OnStateFocus);
        trule[0].reference = this;
        static_throw(RegisterHandler(atomic_cast(trange), trule, lengthof(trange)));
        if (static_try()) {
            if ((_widget = SFYSingleEditWidget::NewInstance(&error)) != null) {
                crule[0].spp = XANDLER_FUNCTION(OnWidgetStyle);
                crule[0].reference = this;
                crule[1].spp = XANDLER_FUNCTION(OnWidgetResult);
                crule[1].reference = this;
                static_throw(_widget->RegisterHandler(atomic_cast(crange), crule, lengthof(crange)));
                if (static_try()) {
                    static_throw(_widget->SetParent(GetThis()));
                    if (static_try()) {
                        SetPropertyTransparent(true);
                        _widget->SetState(true, true, false, false);
                        Relocate();
                    }
                }
            }
            else {
                static_throw(error);
            }
        }
    }
}// SFZSingleEditLabelControl::SFZSingleEditLabelControl //

/*protected virtual */SFZSingleEditLabelControl::~SFZSingleEditLabelControl(Void)
{
}// SFZSingleEditLabelControl::~SFZSingleEditLabelControl //

/*public static */SFZSingleEditLabelControlSmp SFZSingleEditLabelControl::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<SFZSingleEditLabelControl>(Factory(::new SFZSingleEditLabelControl, exception));
}// SFZSingleEditLabelControl::NewInstance //

/*protected virtual */Void SFZSingleEditLabelControl::HandleOperateKey(Void)
{
    SFCError                                    error;

    _widget->StopScroll();
    if ((error = _widget->Edit()) != SFERR_NO_ERROR) {
        _widget->StartScroll();
        InvokeForward(SFXEvent(SFEVT_RESPONDER_RESULT, SFP16_RESULT_ERROR, error), false);
    }
    return;
}// SFZSingleEditLabelControl::HandleOperateKey //

/*protected virtual */Void SFZSingleEditLabelControl::HandleBoundRequest(SFXRectanglePtr rectangle) const
{
    rectangle->Deflate(GetFocusMargin());
    rectangle->SetSize(_widget->GetSuitableBound().GetSize());
    rectangle->Inflate(GetFocusMargin());
    return;
}// SFZSingleEditLabelControl::HandleBoundRequest //

/*protected virtual */Void SFZSingleEditLabelControl::HandleBoundOptimize(SFXRectanglePtr rectangle) const
{
    rectangle->Deflate(GetFocusMargin());
    rectangle->SetSize(_widget->GetSuitableBound(*rectangle).GetSize());
    rectangle->Inflate(GetFocusMargin());
    return;
}// SFZSingleEditLabelControl::HandleBoundOptimize //

/*protected virtual */Void SFZSingleEditLabelControl::HandleBoundVirtual(Void)
{
    Relocate();
    return;
}// SFZSingleEditLabelControl::HandleBoundVirtual //

/*protected virtual */Void SFZSingleEditLabelControl::HandleRenderRequest(SFXGraphicsPtr graphics) const
{
    DrawFocus(graphics, GetLocalBound());
    return;
}// SFZSingleEditLabelControl::HandleRenderRequest //

/*private */XANDLER_IMPLEMENT_VOIDSTATE(SFZSingleEditLabelControl, OnStateFocus, invoker, reason, state)
{
    unused(invoker);
    unused(reason);
    (state) ? (_widget->StartScroll()) : (_widget->StopScroll());
    return;
}// XANDLER_IMPLEMENT_VOIDSTATE(SFZSingleEditLabelControl, OnStateFocus) //

/*private */XANDLER_IMPLEMENT_VOIDSTYLE(SFZSingleEditLabelControl, OnWidgetStyle, invoker, reason)
{
    unused(invoker);
    InvokeBackward(SFXEvent(SFEVT_RESPONDER_STYLE, reason, 0), true);
    return;
}// XANDLER_IMPLEMENT_VOIDSTYLE(SFZSingleEditLabelControl, OnWidgetStyle) //

/*private */XANDLER_IMPLEMENT_VOIDRESULT(SFZSingleEditLabelControl, OnWidgetResult, invoker, reason, result)
{
    unused(invoker);
    switch (reason) {
        case SFP16_RESULT_OK:
            result = GetCurrentValue();
            break;
        default:
            break;
    }
    _widget->StartScroll();
    InvokeForward(SFXEvent(SFEVT_RESPONDER_RESULT, reason, result), false);
    return;
}// XANDLER_IMPLEMENT_VOIDRESULT(SFZSingleEditLabelControl, OnWidgetResult) //

/*private */Void SFZSingleEditLabelControl::Relocate(Void)
{
    _widget->SetRealBound(GetLocalBound().Deflate(GetFocusMargin()));
    return;
}// SFZSingleEditLabelControl::Relocate //
